import config from '@/config';
import Dexie from 'dexie';

interface Asset {
  key: string;
  data: Blob;
  group: string;
  updatedAt: Date;
  frameCount?: number;
  frameWidth?: number;
  frameHeight?: number;
}

interface AssetWithUrl extends Omit<Asset, 'data'> {
  data: string;
}

class AssetManager extends Dexie {
  assets!: Dexie.Table<Asset, string>;

  constructor() {
    super('AssetManager');
    this.version(1).stores({
      assets: 'key, group'
    });
  }

  async getAssetCount(): Promise<number> {
    try {
      const count = await this.assets.count();
      return count;
    } catch (error) {
      console.error('Failed to get asset count:', error);
      return 0;
    }
  }

  async addAsset(
    key: string,
    url: string,
    group: string,
    updatedAt: Date,
    frameCount?: number,
    frameWidth?: number,
    frameHeight?: number
  ): Promise<void> {
    try {
      const response = await fetch(config.server_endpoint + url);
      const blob = await response.blob();
      await this.assets.put({ key, data: blob, group, updatedAt, frameCount, frameWidth, frameHeight });
    } catch (error) {
      console.error(`Failed to add asset ${key}:`, error);
    }
  }

  async getAsset(key: string): Promise<AssetWithUrl | null> {
    try {
      const asset = await this.assets.get(key);
      if (asset) {
        return {
          ...asset,
          data: URL.createObjectURL(asset.data)
        };
      }
    } catch (error) {
      console.error(`Failed to retrieve asset ${key}:`, error);
    }
    return null;
  }

  async getAssetsByGroup(group: string): Promise<AssetWithUrl[]> {
    try {
      const assets = await this.assets.where('group').equals(group).toArray();
      return assets.map(asset => ({
        ...asset,
        data: URL.createObjectURL(asset.data)
      }));
    } catch (error) {
      console.error(`Failed to retrieve assets for group ${group}:`, error);
      return [];
    }
  }

  async clearAssets(): Promise<void> {
    try {
      await this.assets.clear();
    } catch (error) {
      console.error('Failed to clear assets:', error);
    }
  }

  async deleteAsset(key: string): Promise<void> {
    try {
      await this.assets.delete(key);
    } catch (error) {
      console.error(`Failed to delete asset ${key}:`, error);
    }
  }
}

export const useAssetManager = new AssetManager();